@@ -1,11 +1,11 @@
 import inspect
 
-from coworks import Blueprint
+from coworks import Blueprint, jsonify
 
 
 class Admin(Blueprint):
 
-    def get_routes(self):
+    def get_routes(self, pretty=False):
         """Returns the list of entrypoints with signature."""
         app = self._current_app
         routes = {}
@@ -16,10 +16,11 @@ class Admin(Blueprint):
                 doc = inspect.getdoc(function_called)
                 route[http_method] = {
                     'doc': doc if doc else '',
-                    'signature': get_signature(function_called.__class_func__)
+                    'signature': get_signature(function_called.__cws_func__)
                 }
             routes[resource_path] = route
-        return routes
+
+        return jsonify(routes, pretty)
 
     def get_context(self):
         """Returns the calling context."""
