@@ -1,26 +1,19 @@
-import inspect
 import json
 import logging
 import os
 import sys
-import traceback
-from functools import update_wrapper
+from collections import defaultdict
 from threading import Lock
 from typing import Dict, List, Union
 
 from aws_xray_sdk.core import xray_recorder
-from chalice import AuthResponse, BadRequestError, Rate, Cron
+from chalice import BadRequestError, Rate, Cron
 from chalice import Chalice, Blueprint as ChaliceBlueprint
 from requests_toolbelt.multipart import MultipartEncoder
 
-from .config import Config, DEFAULT_WORKSPACE
-from .cws.error import CwsCommandError
-from .mixins import CoworksMixin, AwsSFNSession
-from .utils import begin_xray_subsegment, end_xray_subsegment
-from .utils import class_auth_methods, class_rest_methods, class_attribute, trim_underscores
-
-logger = logging.getLogger()
-logger.setLevel(logging.INFO)
+from .config import Config
+from .mixins import Entry, CoworksMixin, AwsSFNSession
+from .utils import class_auth_methods, make_absolute
 
 
 class Blueprint(CoworksMixin, ChaliceBlueprint):
@@ -47,6 +40,13 @@ class Blueprint(CoworksMixin, ChaliceBlueprint):
     def current_app(self):
         return self._current_app
 
+    @property
+    def logger(self):
+        return self._current_app.logger
+
+    def entry(self, route):
+        return self.current_app.entry(route)
+
     def deferred_init(self, workspace):
         for func in self.before_first_activation_funcs:
             self.current_app.before_first_activation(func)
@@ -54,6 +54,8 @@ class Blueprint(CoworksMixin, ChaliceBlueprint):
             self.current_app.before_activation(func)
         for func in self.after_activation_funcs:
             self.current_app.after_activation(func)
+        for func in self.handle_exception_funcs:
+            self.current_app.handle_exception_funcs(func)
 
 
 class TechMicroService(CoworksMixin, Chalice):
@@ -72,6 +74,8 @@ class TechMicroService(CoworksMixin, Chalice):
         """
         name = name or self.__class__.__name__.lower()
 
+        self.logger = self.create_logger(name, **kwargs)
+
         self.configs = configs or [Config()]
         if type(self.configs) is not list:
             self.configs = [configs]
@@ -91,14 +95,13 @@ class TechMicroService(CoworksMixin, Chalice):
         self._got_first_activation = False
         self._before_activation_lock = Lock()
 
+        self.__auth__ = None
         self.entries = None
         self.sfn_call = False
 
         if "pytest" in sys.modules:
             xray_recorder.configure(context_missing="LOG_ERROR")
 
-        self.__global_auth__ = None
-
     @property
     def name(self):
         return self.app_name
@@ -107,65 +110,18 @@ class TechMicroService(CoworksMixin, Chalice):
     def ms_type(self):
         return 'tech'
 
+    @staticmethod
+    def create_logger(name, **kwargs):
+        logger = logging.getLogger(name)
+        if 'debug' in kwargs:
+            logger.setLevel(logging.INFO)
+        return logger
+
     def deferred_init(self, workspace):
         if self.entries is None:
-            url_prefix = class_attribute(self, 'url_prefix', '')
-            self._init_routes(workspace=workspace, url_prefix=url_prefix)
-            for deferred_init in self.deferred_inits:
-                deferred_init(workspace)
-            for blueprint in self.iter_blueprints():
-                blueprint.deferred_init(workspace)
+            self.entries = defaultdict(Entry)
 
-    def register_blueprint(self, blueprint: Blueprint, authorizer=None, url_prefix=None, hide_routes=False,
-                           **kwargs):
-        """ Register a :class:`Blueprint` on the microservice.
-
-        :param blueprint:
-        :param authorizer:
-        :param url_prefix:
-        :param hide_routes:
-        :param kwargs:
-        :return:
-        """
-        if 'name_prefix' not in kwargs:
-            kwargs['name_prefix'] = blueprint.name
-        if 'url_prefix' not in kwargs:
-            kwargs['url_prefix'] = f"/{blueprint.name}"
-
-        if url_prefix in self.blueprints:
-            raise NameError(f"A blueprint is already defined for the {url_prefix} prefix.")
-        self.blueprints[url_prefix] = blueprint
-
-        if not hide_routes:
-            blueprint.__auth__ = self._create_auth_proxy(self, authorizer) if authorizer else None
-
-        def deferred(workspace):
-            if not hide_routes:
-                self._init_routes(workspace=workspace, component=blueprint, url_prefix=url_prefix)
-            Chalice.register_blueprint(self, blueprint, url_prefix=url_prefix)
-
-        self.deferred_inits.append(deferred)
-
-    def iter_blueprints(self):
-        return self.blueprints.values()
-
-    def execute(self, command, *, project_dir, module, service=None, workspace, output=None, error=None, **kwargs):
-        """Executes a coworks command."""
-        from coworks.cws.client import ProjectConfig
-
-        project_config = ProjectConfig(command, project_dir)
-        if not service:
-            service = self.name
-        cmd = project_config.get_command(self, module, service, workspace)
-        if not cmd:
-            raise CwsCommandError(f"The command {command} was not added to the microservice {self.name}.\n")
-
-        client_params = {'project_dir': project_dir, 'module': module, 'service': service, 'workspace': workspace}
-        complemented_args = project_config.missing_options(**client_params, **kwargs)
-        cmd.execute(output=output, error=error, **client_params, **complemented_args)
-
-    def _init_routes(self, *, workspace=DEFAULT_WORKSPACE, component=None, url_prefix=''):
-        if self.config is None:
+            # Set workspace config
             for conf in self.configs:
                 if conf.workspace == workspace:
                     self.config = conf
@@ -173,120 +129,116 @@ class TechMicroService(CoworksMixin, Chalice):
             if self.config is None:
                 self.config = self.configs[0]
 
-        if component is None:
-            component = self
+            # Initializes routes
+            if self.config.auth:
+                self.__auth__ = self._create_auth_proxy(self.config.auth)
+            else:
+                auth_fun = class_auth_methods(self)
+                if auth_fun:
+                    self.__auth__ = self._create_auth_proxy(auth_fun)
+            self._init_routes(self, authorizer=self.__auth__)
 
-        # External authorizer has priority (forced)
-        if self.config.auth:
-            if self.__global_auth__ is None:
-                self.__global_auth__ = self._create_auth_proxy(self, self.config.auth)
-            auth = component.__auth__ = self.__global_auth__
-        else:
-            auth = class_auth_methods(component)
-            if auth and component.__auth__ is None:
-                auth = TechMicroService._create_auth_proxy(component, auth)
-                component.__auth__ = auth
-            auth = component.__auth__
+            for deferred_init in self.deferred_inits:
+                deferred_init(workspace)
+            for blueprint in self.iter_blueprints():
+                blueprint.deferred_init(workspace)
 
-        # Adds entrypoints
-        if self.entries is None:
-            self.entries = {}
-        methods = class_rest_methods(component)
-        for method, func in methods:
-            if getattr(func, '__cws_hidden', False):
+    def register_blueprint(self, blueprint: Blueprint, url_prefix=None, authorizer=None, hide_routes=False):
+        """ Register a :class:`Blueprint` on the microservice.
+
+        :param blueprint: blueprint to register.
+        :param url_prefix: url prefix for the routes added (must be unique for each blueprints).
+        :param authorizer: authorizer for this blueprint.
+        :param hide_routes:
+        :return:
+        """
+        if url_prefix in self.blueprints:
+            raise NameError(f"A blueprint is already defined for the {url_prefix} prefix.")
+        self.blueprints[url_prefix] = blueprint
+
+        def deferred(workspace):
+            auth = self._create_auth_proxy(authorizer) if authorizer else self.__auth__
+            blueprint._init_routes(self, url_prefix=url_prefix, authorizer=auth, hide_routes=hide_routes)
+            super(TechMicroService, self).register_blueprint(blueprint, url_prefix=url_prefix)
+
+        self.deferred_inits.append(deferred)
+
+    def entry(self, route):
+        route = make_absolute(route)
+        route_pathes = route.split('/')
+        for entry, result in self.entries.items():
+            entry_pathes = entry.split('/')
+            if len(route_pathes) != len(entry_pathes):
                 continue
 
-            # Get function's route
-            if func.__name__ == method:
-                route = f"{url_prefix}"
-            else:
-                name = func.__name__[len(method) + 1:]
-                name = trim_underscores(name)  # to allow several functions with same route but different args
-                name = name.replace('_', '/')
-                route = f"{url_prefix}/{name}" if url_prefix else f"{name}"
+            found = True
+            for index, path in enumerate(entry_pathes):
+                if index < len(route_pathes):
+                    if path.startswith('{') or path == route_pathes[index]:
+                        continue
+                found = False
+                break
 
-            # Get parameters
-            args = inspect.getfullargspec(func).args[1:]
-            defaults = inspect.getfullargspec(func).defaults
-            varkw = inspect.getfullargspec(func).varkw
-            if defaults:
-                len_defaults = len(defaults)
-                for index, arg in enumerate(args[:-len_defaults]):
-                    route = route + f"/{{_{index}}}" if route else f"{{_{index}}}"
-                kwarg_keys = args[-len_defaults:]
-            else:
-                for index, arg in enumerate(args):
-                    route = route + f"/{{_{index}}}" if route else f"{{_{index}}}"
-                kwarg_keys = {}
+            if found:
+                return result
+        return None
 
-            proxy = component._create_rest_proxy(func, kwarg_keys, args, varkw)
+    def iter_blueprints(self):
+        return self.blueprints.values()
 
-            # complete all entries
-            if not route.startswith('/'):
-                route = '/' + route
-            self.route(f"{route}", methods=[method.upper()], authorizer=auth, cors=self.config.cors,
-                       content_types=list(self.config.content_type))(proxy)
-            if url_prefix:
-                self.entries[f"{url_prefix}{route}"] = (method.upper(), func)
-            else:
-                self.entries[f"{route}"] = (method.upper(), func)
+    def execute(self, command, *, project_dir, module=None, service=None, workspace, output=None, error=None,
+                **options):
+        """Executes a coworks command."""
+        from .cws.error import CwsCommandError
+        from .cws.client import ProjectConfig  # only available from client module
 
-    @staticmethod
-    def _create_auth_proxy(component, auth_method):
+        module = __name__ if module is None else module
+        service = self.name if service is None else service
 
-        def proxy(auth_activation):
-            subsegment = begin_xray_subsegment(f"auth microservice")
-            try:
-                auth = auth_method(component, auth_activation)
-                if subsegment:
-                    subsegment.put_metadata('result', auth)
-            except Exception as e:
-                logger.info(f"Exception : {str(e)}")
-                traceback.print_exc()
-                if subsegment:
-                    subsegment.add_exception(e, traceback.extract_stack())
-                raise BadRequestError(str(e))
-            finally:
-                end_xray_subsegment()
-
-            if type(auth) is bool:
-                if auth:
-                    return AuthResponse(routes=['*'], principal_id='user')
-                return AuthResponse(routes=[], principal_id='user')
-            elif type(auth) is list:
-                return AuthResponse(routes=auth, principal_id='user')
-            return auth
-
-        proxy = update_wrapper(proxy, auth_method)
-        proxy.__name__ = 'app'
-        return component.authorizer(name='auth')(proxy)
+        project_config = ProjectConfig(project_dir)
+        if not project_config.params:
+            self.logger.debug(f"No project parameters defined (check {project_config.project_file} file exists).")
+        service_config = project_config.get_service_config(module, service, workspace)
+        cmd = service_config.get_command(command, self)
+        if not cmd:
+            raise CwsCommandError(f"The command {command} was not added to the microservice {self.name}.\n")
+        command_options = service_config.get_command_options(command)
+        execution_params = {**command_options, **options}
+        cmd.execute(output=output, error=error, **execution_params)
 
     def __call__(self, event, context):
         """Lambda handler."""
-        self.do_before_first_activation()
-        self.do_before_activation()
-        res = self.handler(event, context)
-        self.do_after_activation()
-        return res
+        try:
+            self.do_before_first_activation(event, context)
+            self.do_before_activation(event, context)
+            response = self.handler(event, context)
+            return self.do_after_activation(response)
+        except Exception as e:
+            print(f"exception: {e}")
+            response = self.do_handle_exception(e)
+            if response is None:
+                raise
+            return response
 
     def handler(self, event, context):
         """Main microservice entry point."""
 
         # authorization call
         if event.get('type') == 'TOKEN':
-            if self.debug:
-                print(f"Calling {self.name} for authorization")
+            self.logger.debug(f"Calling {self.name} for authorization")
 
-            if self.__auth__:
-                return self.__auth__(event, context)
+            route = event.get('methodArn').split('/', 3)[-1]
+            authorizer = self.entry(f'/{route}').authorizer
+            if authorizer:
+                return authorizer(event, context)
 
-            print(f"Undefined authorization method for {self.name} ")
+            self.logger.debug(f"Undefined authorization method for {self.name} ")
             return 'Unauthorized', 403
 
         # step function call
         if event.get('type') == 'CWS_SFN':
             if self.debug:
-                print(f"Calling {self.name} by step function")
+                self.logger.debug(f"Calling {self.name} by step function")
 
             self.sfn_call = True
             content_type = event['headers']['Content-Type']
@@ -300,10 +252,8 @@ class TechMicroService(CoworksMixin, Chalice):
                     event['body'] = multi_parts.to_string()
             else:
                 raise BadRequestError(f"Undefined content type {content_type} for Step Function call")
-        else:
-            if self.debug:
-                print(f"Calling {self.name} with event {event}")
 
+        self.logger.debug(f"Calling {self.name} with event {event}")
         res = super().__call__(event, context)
 
         if self.sfn_call:
@@ -314,9 +264,7 @@ class TechMicroService(CoworksMixin, Chalice):
             except json.JSONDecodeError:
                 pass
 
-        if self.debug:
-            print(f"Call {self.name} returns {res}")
-
+        self.logger.debug(f"Call {self.name} returns {res}")
         return res
 
     def deferred(self, f):
@@ -330,7 +278,7 @@ class TechMicroService(CoworksMixin, Chalice):
         self.deferred_inits.append(f)
         return f
 
-    def do_before_first_activation(self):
+    def do_before_first_activation(self, event, context):
         """Calls all before first activation functions."""
         if self._got_first_activation:
             return
@@ -342,18 +290,28 @@ class TechMicroService(CoworksMixin, Chalice):
 
             if not self._got_first_activation:
                 for func in self.before_first_activation_funcs:
-                    func()
+                    func(event, context)
                 self._got_first_activation = True
 
-    def do_before_activation(self):
+    def do_before_activation(self, event, context):
         """Calls all before activation functions."""
         for func in self.before_activation_funcs:
-            func()
+            func(event, context)
 
-    def do_after_activation(self):
+    def do_after_activation(self, response):
         """Calls all after activation functions."""
-        for func in self.after_activation_funcs:
-            func()
+        for func in reversed(self.after_activation_funcs):
+            resp = func(response)
+            if resp is not None:
+                response = resp
+        return response
+
+    def do_handle_exception(self, exc):
+        """Calls all exception handlers."""
+        for func in reversed(self.handle_exception_funcs):
+            resp = func(exc)
+            if resp is not None:
+                return resp
 
 
 class BizFactory(TechMicroService):
@@ -456,7 +414,7 @@ class BizMicroService(TechMicroService):
         try:
             default_data = next(c.data for c in self.configs if c.workspace == workspace)
         except StopIteration:
-            print(f"No configuration found for workspace {workspace} in {self.configs}")
+            self.logger.debug(f"No configuration found for workspace {workspace} in {self.configs}")
             default_data = {}
         except KeyError:
             default_data = {}
@@ -470,7 +428,7 @@ class BizMicroService(TechMicroService):
     def handler(self, event, context):
         if 'detail-type' in event and event['detail-type'] == 'Scheduled Event':
             if self.debug:
-                print(f"Trigger: {self.biz_factory.sfn_name}")
+                self.logger.debug(f"Trigger: {self.biz_factory.sfn_name}")
             return self.biz_factory.invoke(self.get_default_data())
 
         return super().handler(event, context)
