@@ -1,19 +1,46 @@
 import cgi
+import inspect
 import io
 import json
 import os
-import sys
 import traceback
-from functools import update_wrapper
 import urllib
+from functools import update_wrapper, partial
 
 import boto3
+from aws_xray_sdk.core import xray_recorder
 from botocore.exceptions import BotoCoreError
-from chalice import BadRequestError, Response
+from chalice import AuthResponse, BadRequestError, Response
 from requests_toolbelt.multipart import MultipartDecoder
 
-from coworks.cws.error import CwsError
-from coworks.utils import begin_xray_subsegment, end_xray_subsegment
+from coworks.error import CwsError
+from .utils import HTTP_METHODS, class_auth_methods, class_http_methods, trim_underscores, make_absolute
+
+
+class EntryPoint:
+
+    def __init__(self, component, auth, fun):
+        self.component = component
+        self.auth = auth
+        self.fun = fun
+
+
+class Entry(dict):
+
+    @property
+    def authorizer(self):
+        for method in HTTP_METHODS:
+            m = self.get(method.upper())
+            if m:
+                return m.auth
+
+    def call_get(self, *args, **kwargs):
+        method = self["GET"]
+        return method.fun(method.component, *args, **kwargs)
+
+    def call_post(self, *args, **kwargs):
+        method = self["POST"]
+        return method.fun(method.component, *args, **kwargs)
 
 
 class CoworksMixin:
@@ -21,7 +48,6 @@ class CoworksMixin:
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.debug = kwargs.pop('debug', False)
-        self.__auth__ = None
 
         # A list of functions that will be called at the first activation.
         # To register a function, use the :meth:`before_first_request` decorator.
@@ -31,8 +57,14 @@ class CoworksMixin:
         # To register a function, use the :meth:`before_activation` decorator.
         self.before_activation_funcs = []
 
+        # A list of functions that will be called after each activation.
+        # To register a function, use the :meth:`after_activation` decorator.
         self.after_activation_funcs = []
 
+        # A list of functions that will be called in case of exception.
+        # To register a function, use the :meth:`handle_exception` decorator.
+        self.handle_exception_funcs = []
+
         self.aws_s3_sfn_data_session = AwsS3Session(env_var_access_key="AWS_RUN_ACCESS_KEY_ID",
                                                     env_var_secret_key="AWS_RUN_SECRET_KEY",
                                                     env_var_region="AWS_RUN_REGION")
@@ -45,48 +77,124 @@ class CoworksMixin:
 
         May be used as a decorator.
 
-        The function will be called without any arguments and its return value is ignored.
+        The function will be called with event and context positional arguments and its return value is ignored.
         """
 
         self.before_first_activation_funcs.append(f)
         return f
 
     def before_activation(self, f):
-        """Registers a function to run before each activation of the microservice.
+        """Registers a function to called before each activation of the microservice.
         :param f:  Function added to the list.
         :return: None.
 
         May be used as a decorator.
 
-        The function will be called without any arguments and its return value is ignored.
+        The function will be called with event and context positional arguments and its return value is ignored.
         """
 
         self.before_activation_funcs.append(f)
         return f
 
     def after_activation(self, f):
-        """Registers a function to be run after each activation of the microservice.
+        """Registers a function to be called after each activation of the microservice.
 
         May be used as a decorator.
 
-        The function will be called without any arguments and its return value is ignored.
+        The function will be called with the response and its return value is the final response.
         """
 
         self.after_activation_funcs.append(f)
         return f
 
+    def handle_exception(self, f):
+        """Registers a function to be called in case of exception.
+
+        May be used as a decorator.
+
+        The function will be called with the exception and eventually a response to return.
+        """
+
+        self.handle_exception_funcs.append(f)
+        return f
+
+    def _init_routes(self, app, *, url_prefix='', authorizer=None, hide_routes=False):
+        # External authorizer has priority (forced)
+        if authorizer is None:
+            auth_fun = app.config.auth if app.config.auth else class_auth_methods(self)
+            auth = self._create_auth_proxy(auth_fun) if auth_fun else None
+        else:
+            auth = authorizer
+
+        # Adds entrypoints
+        methods = class_http_methods(self)
+        for method, func in methods:
+
+            # Get function's route
+            if func.__name__ == method:
+                route = f"{url_prefix}"
+            else:
+                name = func.__name__[len(method) + 1:]
+                name = trim_underscores(name)  # to allow several functions with same route but different args
+                name = name.replace('_', '/')
+                route = f"{url_prefix}/{name}" if url_prefix else f"{name}"
+
+            # Get parameters
+            args = inspect.getfullargspec(func).args[1:]
+            defaults = inspect.getfullargspec(func).defaults
+            varkw = inspect.getfullargspec(func).varkw
+            if defaults:
+                len_defaults = len(defaults)
+                for index, arg in enumerate(args[:-len_defaults]):
+                    route = route + f"/{{_{index}}}" if route else f"{{_{index}}}"
+                kwarg_keys = args[-len_defaults:]
+            else:
+                for index, arg in enumerate(args):
+                    route = route + f"/{{_{index}}}" if route else f"{{_{index}}}"
+                kwarg_keys = {}
+
+            proxy = self._create_rest_proxy(func, kwarg_keys, args, varkw)
+
+            # complete all entries
+            route = make_absolute(route)
+            app.entries[route][method.upper()] = EntryPoint(self, auth, func)
+            if not hide_routes and not getattr(func, '__cws_hidden', False):
+                app.route(f"{route}", methods=[method.upper()], authorizer=auth, cors=app.config.cors,
+                          content_types=list(app.config.content_type))(proxy)
+
+    def _create_auth_proxy(self, auth_method):
+
+        # @xray_recorder.capture('auth')
+        def proxy(auth_activation):
+            subsegment = xray_recorder.current_subsegment()
+            try:
+                auth = auth_method(self, auth_activation)
+                if subsegment:
+                    subsegment.put_metadata('result', auth)
+            except Exception as e:
+                self.logger.info(f"Exception : {str(e)}")
+                traceback.print_exc()
+                if subsegment:
+                    subsegment.add_exception(e, traceback.extract_stack())
+                raise BadRequestError(str(e))
+
+            if type(auth) is bool:
+                if auth:
+                    return AuthResponse(routes=['*'], principal_id='user')
+                return AuthResponse(routes=[], principal_id='user')
+            elif type(auth) is list:
+                return AuthResponse(routes=auth, principal_id='user')
+            return auth
+
+        proxy = update_wrapper(proxy, auth_method)
+        proxy.__name__ = 'app'
+        return self.authorizer(name='auth')(proxy)
+
     def _create_rest_proxy(self, func, kwarg_keys, args, varkw):
         original_app_class = self.__class__
 
         def proxy(**kws):
-            if self.debug:
-                print(f"Calling {func} for {self}")
-
-            subsegment = begin_xray_subsegment(f"{func.__name__} microservice")
             try:
-                if subsegment:
-                    subsegment.put_metadata('headers', self.current_request.headers, "Coworks")
-
                 # Renames positional parameters (index added in label)
                 kwargs = {}
                 for kw, value in kws.items():
@@ -146,30 +254,28 @@ class CoworksMixin:
                             value = req.query_params.getlist(k)
                             add_param(k, value if len(value) > 1 else value[0])
                         kwargs = dict(**kwargs, **params)
-
-                if self.debug:
-                    print(f"Calling {self} with event {kwargs}")
+                else:
+                    if not args and (req.raw_body or req.query_params):
+                        raise BadRequestError(f"TypeError: got an unexpected arguments")
 
                 # chalice is changing class for local server for threading reason (why not mixin..?)
                 self_class = self.__class__
                 if self_class != original_app_class:
                     self.__class__ = original_app_class
+
                 resp = func(self, **kwargs)
                 self.__class__ = self_class
-
                 return _convert_response(resp)
 
             except Exception as e:
-                print(f"Exception : {str(e)}")
-                traceback.print_exc()
-                if subsegment:
-                    subsegment.add_exception(e, traceback.extract_stack())
-                raise BadRequestError(str(e))
+                raise
             finally:
-                end_xray_subsegment()
+                subsegment = xray_recorder.current_subsegment()
+                if subsegment:
+                    subsegment.add_error_flag()
 
         proxy = update_wrapper(proxy, func)
-        proxy.__class_func__ = func
+        proxy.__cws_func__ = update_wrapper(partial(func, self), func)
         return proxy
 
     def _get_multipart_content(self, part):
@@ -352,7 +458,8 @@ class Boto3Mixin:
                 try:
                     self.__session__ = boto3.Session(access_key, secret_key, region_name=region_name)
                 except Exception:
-                    raise CwsError(f"Cannot create session for key {access_key}, secret {secret_key} and region {region_name}")
+                    raise CwsError(
+                        f"Cannot create session for key {access_key}, secret {secret_key} and region {region_name}")
         return self.__session__
 
 
