@@ -1,10 +1,32 @@
 import importlib
 import inspect
 import os
+import platform
 import sys
+import json
+from functools import wraps
+from threading import Thread
 
 from aws_xray_sdk.core import xray_recorder
 
+HTTP_METHODS = ['get', 'post', 'put', 'delete', 'patch', 'options']
+
+
+def make_absolute(route):
+    if not route.startswith('/'):
+        route = '/' + route
+    return route
+
+
+def jsonify(result, pretty=False, indent=None, separators=None):
+    if pretty:
+        indent = indent or 4
+        separators = separators or (",", ": ")
+    else:
+        separators = separators or (",", ":")
+
+    return json.dumps(result, indent=indent, separators=separators)
+
 
 def import_attr(module, attr, cwd='.'):
     if cwd not in sys.path:
@@ -26,24 +48,16 @@ def class_auth_methods(obj):
     return None
 
 
-def class_rest_methods(obj):
+def class_http_methods(obj):
     """Returns the list of methods from the class."""
     methods = inspect.getmembers(obj.__class__, lambda x: inspect.isfunction(x))
 
     res = []
     for name, func in methods:
-        if name == 'get' or name.startswith('get_'):
-            res.append(('get', func))
-        elif name == 'post' or name.startswith('post_'):
-            res.append(('post', func))
-        elif name == 'put' or name.startswith('put_'):
-            res.append(('put', func))
-        elif name == 'delete' or name.startswith('delete_'):
-            res.append(('delete', func))
-        elif name == 'patch' or name.startswith('patch_'):
-            res.append(('patch', func))
-        elif name == 'options' or name.startswith('options_'):
-            res.append(('options', func))
+        for method in HTTP_METHODS:
+            if name == method or name.startswith(f'{method}_'):
+                res.append((method, func))
+                break
     return res
 
 
@@ -84,3 +98,20 @@ def begin_xray_subsegment(subsegment_name):
 def end_xray_subsegment():
     if xray_recorder.in_subsegment().subsegment is not None:
         return xray_recorder.end_subsegment()
+
+
+def threaded(f):
+    @wraps(f)
+    def threaded_func(*args, **kwargs):
+        thread = Thread(target=f, args=args, kwargs=kwargs)
+        thread.start()
+
+    return threaded_func
+
+
+def get_system_info():
+    python_info = f"python {sys.version_info[0]}.{sys.version_info[1]}.{sys.version_info[2]}"
+    platform_system = platform.system().lower()
+    platform_release = platform.release()
+    platform_info = f"{platform_system} {platform_release}"
+    return f"{python_info}, {platform_info}"
