@@ -17,6 +17,9 @@ CONFIG = Config(
     workspace="dev"
 )
 
-app = SimpleMicroService(ms_name='test', configs=[CONFIG])
+app = SimpleMicroService(name='test', configs=[CONFIG])
 CwsRunner(app)
 CwsTerraformWriter(app, name='export')
+
+if __name__ == '__main__':
+    app.execute('run', project_dir='.', workspace='dev', module='quickstart')
