@@ -1,7 +1,7 @@
 from coworks import BizFactory, Every
-from coworks.cws.writer.sfn import StepFunctionWriter
+from coworks.cws.sfn import StepFunctionWriter
 
-biz = BizFactory(app_name='biz')
+biz = BizFactory(ms_name='biz')
 reactor1 = biz.create('small', 'often', Every(1, Every.DAYS))
 reactor2 = biz.create('complete', 'often', Every(1, Every.DAYS))
 reactor3 = biz.create('complete', 'rarely', Every(365, Every.DAYS))
